# -------------------------------------------------------------------------
#
#        D E S T I N A T I O N   S Y S T E M
#
# -------------------------------------------------------------------------
import pathlib

from .destinations.interfaces import DestinationDecorator, DeliveryContextIF, Destination
from .destinations.tar import TarArchiver
from .destinations.local import LocalDestination
from .destinations.checksum import ChecksumDecorator
from .destinations.sharedweb import SharedWebDestination
from .destinations.fetchfile import FetchFileGenerator


class DestinationBuilder:
    """
    Facilitates building a stack of destination and its decorators.
    """

    def __init__(self, context: DeliveryContextIF):
        self.context = context
        self._destination = None

    def local(self, path: pathlib.Path, prefix: str = ""):
        """Base the delivery on a local destination with the given path"""
        self._destination = LocalDestination(self.context, path / prefix)
        return self

    def web(self, prefix: str = ""):
        """Base the delivery on a shared web destination"""
        self._destination = SharedWebDestination(self.context, prefix)
        return self

    def checksums(self):
        "Add a SHA1SUMS file to the delivery"
        self._destination = ChecksumDecorator(self._destination)
        return self

    def fetchfile(self):
        """Add the fetch-all.sh script to the delivery."""
        self._destination = FetchFileGenerator(self._destination)
        return self

    def tar(self):
        """Add the tar decorator to the destination"""
        self._destination = TarArchiver(self.context, self._destination)
        return self

    def build(self):
        """Create the destination"""
        return self._destination
